@@ -52,6 +52,7 @@ class Revision(models.Model):
     )
 
     date_created = models.DateTimeField(auto_now_add=True,
+                                        db_index=True,
                                         verbose_name=_("date created"),
                                         help_text="The date and time this revision was created.")
 
@@ -94,6 +95,10 @@ class Revision(models.Model):
         """Returns a unicode representation."""
         return ", ".join(force_text(version) for version in self.version_set.all())
 
+    #Meta
+    class Meta:
+        app_label = 'reversion'
+
 
 def has_int_pk(model):
     """Tests whether the given model has an integer primary key."""
@@ -161,7 +166,9 @@ class Version(models.Model):
                 result[field.name] = field.value_from_object(obj)
             result.update(object_version.m2m_data)
             # Add parent data.
-            for parent_class, field in obj._meta.parents.items():
+            for parent_class, field in obj._meta.concrete_model._meta.parents.items():
+                if obj._meta.proxy and parent_class == obj._meta.concrete_model:
+                    continue
                 content_type = ContentType.objects.get_for_model(parent_class)
                 if field:
                     parent_id = force_text(getattr(obj, field.attname))
@@ -186,6 +193,10 @@ class Version(models.Model):
         """Returns a unicode representation."""
         return self.object_repr
 
+    #Meta
+    class Meta:
+        app_label = 'reversion'
+
 
 # Version management signals.
 pre_revision_commit = Signal(providing_args=["instances", "revision", "versions"])
