@@ -3,15 +3,16 @@
 from __future__ import unicode_literals
 
 import operator, sys
-from functools import wraps, reduce
+from functools import wraps, reduce, partial
 from threading import local
 from weakref import WeakValueDictionary
+import copy
 
 from django.contrib.contenttypes.models import ContentType
 from django.core import serializers
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.signals import request_finished
-from django.db import models, DEFAULT_DB_ALIAS, connection, transaction
+from django.db import models, connection, transaction
 from django.db.models import Q, Max, get_model
 from django.db.models.query import QuerySet
 from django.db.models.signals import post_save
@@ -21,28 +22,28 @@ from reversion.models import Revision, Version, has_int_pk, pre_revision_commit,
 
 
 class VersionAdapter(object):
-    
+
     """Adapter class for serializing a registered model."""
-    
+
     # Fields to include in the serialized data.
     fields = ()
-    
+
     # Fields to exclude from the serialized data.
     exclude = ()
-    
+
     # Foreign key relationships to follow when saving a version of this model.
     follow = ()
-    
+
     # The serialization format to use.
     format = "json"
-    
+
     def __init__(self, model):
         """Initializes the version adapter."""
         self.model = model
-        
+
     def get_fields_to_serialize(self):
         """Returns an iterable of field names to serialize in the version data."""
-        opts = self.model._meta
+        opts = self.model._meta.concrete_model._meta
         fields = self.fields or (field.name for field in opts.local_fields + opts.local_many_to_many)
         fields = (opts.get_field(field) for field in fields if not field in self.exclude)
         for field in fields:
@@ -50,7 +51,7 @@ class VersionAdapter(object):
                 yield field.name
             else:
                 yield field.attname
-    
+
     def get_followed_relations(self, obj):
         """Returns an iterable of related models that should be included in the revision data."""
         for relationship in self.follow:
@@ -78,11 +79,11 @@ class VersionAdapter(object):
                     relationship = relationship,
                     related = related,
                 ))
-    
+
     def get_serialization_format(self):
         """Returns the serialization format to use."""
         return self.format
-        
+
     def get_serialized_data(self, obj):
         """Returns a string of serialized data for the given obj."""
         return serializers.serialize(
@@ -90,11 +91,10 @@ class VersionAdapter(object):
             (obj,),
             fields = list(self.get_fields_to_serialize()),
         )
-        
+
     def get_version_data(self, obj, db=None):
         """Creates the version data to be saved to the version model."""
         object_id = force_text(obj.pk)
-        db = db or DEFAULT_DB_ALIAS
         content_type = ContentType.objects.db_manager(db).get_for_model(obj)
         if has_int_pk(obj.__class__):
             object_id_int = int(obj.pk)
@@ -111,20 +111,20 @@ class VersionAdapter(object):
 
 
 class RevisionManagementError(Exception):
-    
+
     """Exception that is thrown when something goes wrong with revision managment."""
 
-          
+
 class RevisionContextManager(local):
-    
+
     """Manages the state of the current revision."""
-    
+
     def __init__(self):
         """Initializes the revision state."""
         self.clear()
         # Connect to the request finished signal.
         request_finished.connect(self._request_finished_receiver)
-    
+
     def clear(self):
         """Puts the revision manager back into its default state."""
         self._objects = {}
@@ -135,31 +135,31 @@ class RevisionContextManager(local):
         self._meta = []
         self._ignore_duplicates = False
         self._db = None
-    
+
     def is_active(self):
         """Returns whether there is an active revision for this thread."""
         return bool(self._stack)
-    
+
     def is_managing_manually(self):
         """Returns whether this revision context has manual management enabled."""
         self._assert_active()
         return self._stack[-1]
-    
+
     def _assert_active(self):
         """Checks for an active revision, throwning an exception if none."""
         if not self.is_active():
             raise RevisionManagementError("There is no active revision for this thread")
-        
+
     def start(self, manage_manually=False):
         """
         Begins a revision for this thread.
-        
+
         This MUST be balanced by a call to `end`.  It is recommended that you
         leave these methods alone and instead use the revision context manager
         or the `create_revision` decorator.
         """
         self._stack.append(manage_manually)
-    
+
     def end(self):
         """Ends a revision for this thread."""
         self._assert_active()
@@ -188,11 +188,11 @@ class RevisionContextManager(local):
         """Marks this revision as broken, so should not be commited."""
         self._assert_active()
         self._is_invalid = True
-        
+
     def is_invalid(self):
         """Checks whether this revision is invalid."""
         return self._is_invalid
-    
+
     def add_to_context(self, manager, obj, version_data):
         """Adds an object to the current revision."""
         self._assert_active()
@@ -206,7 +206,7 @@ class RevisionContextManager(local):
     def get_db(self):
         """Returns the current DB alias being used."""
         return self._db
-    
+
     def set_db(self, db):
         """Sets the DB alias to use."""
         self._db = db
@@ -215,39 +215,39 @@ class RevisionContextManager(local):
         """Sets the current user for the revision."""
         self._assert_active()
         self._user = user
-    
+
     def get_user(self):
         """Gets the current user for the revision."""
         self._assert_active()
         return self._user
-        
+
     def set_comment(self, comment):
         """Sets the comments for the revision."""
         self._assert_active()
         self._comment = comment
-    
+
     def get_comment(self):
         """Gets the current comment for the revision."""
         self._assert_active()
         return self._comment
-        
+
     def add_meta(self, cls, **kwargs):
         """Adds a class of meta information to the current revision."""
         self._assert_active()
         self._meta.append((cls, kwargs))
-    
+
     def set_ignore_duplicates(self, ignore_duplicates):
         """Sets whether to ignore duplicate revisions."""
         self._assert_active()
         self._ignore_duplicates = ignore_duplicates
-        
+
     def get_ignore_duplicates(self):
         """Gets whether to ignore duplicate revisions."""
         self._assert_active()
         return self._ignore_duplicates
-    
+
     # Signal receivers.
-    
+
     def _request_finished_receiver(self, **kwargs):
         """
         Called at the end of a request, ensuring that any open revisions
@@ -256,13 +256,13 @@ class RevisionContextManager(local):
         """
         while self.is_active():
             self.end()
-    
+
     # High-level context management.
-    
+
     def create_revision(self, manage_manually=False):
         """
         Marks up a block of code as requiring a revision to be created.
-        
+
         The returned context manager can also be used as a decorator.
         """
         return RevisionContext(self, manage_manually)
@@ -276,11 +276,11 @@ class RevisionContext(object):
         """Initializes the revision context."""
         self._context_manager = context_manager
         self._manage_manually = manage_manually
-    
+
     def __enter__(self):
         """Enters a block of revision management."""
         self._context_manager.start(self._manage_manually)
-        
+
     def __exit__(self, exc_type, exc_value, traceback):
         """Leaves a block of revision management."""
         try:
@@ -288,7 +288,7 @@ class RevisionContext(object):
                 self._context_manager.invalidate()
         finally:
             self._context_manager.end()
-        
+
     def __call__(self, func):
         """Allows this revision context to be used as a decorator."""
         @wraps(func)
@@ -303,28 +303,28 @@ revision_context_manager = RevisionContextManager()
 
 
 class RegistrationError(Exception):
-    
+
     """Exception thrown when registration with django-reversion goes wrong."""
-   
-   
+
+
 class RevisionManager(object):
-    
+
     """Manages the configuration and creation of revisions."""
-    
+
     _created_managers = WeakValueDictionary()
-    
+
     @classmethod
     def get_created_managers(cls):
         """Returns all created revision managers."""
         return list(cls._created_managers.items())
-    
+
     @classmethod
     def get_manager(cls, manager_slug):
         """Returns the manager with the given slug."""
         if manager_slug in cls._created_managers:
             return cls._created_managers[manager_slug]
         raise RegistrationError("No revision manager exists with the slug %r" % manager_slug)
-    
+
     def __init__(self, manager_slug, revision_context_manager=revision_context_manager):
         """Initializes the revision manager."""
         # Check the slug is unique for this revision manager.
@@ -336,6 +336,8 @@ class RevisionManager(object):
         self._manager_slug = manager_slug
         self._registered_models = {}
         self._revision_context_manager = revision_context_manager
+        self._eager_signals = {}
+        self._signals = {}
         # Proxies to common context methods.
         self._revision_context = revision_context_manager.create_revision()
 
@@ -354,7 +356,7 @@ class RevisionManager(object):
         manager.
         """
         return self._registration_key_for_model(model) in self._registered_models
-    
+
     def get_registered_models(self):
         """Returns an iterable of all registered models."""
         return [
@@ -362,31 +364,35 @@ class RevisionManager(object):
             for key
             in self._registered_models.keys()
         ]
-        
-    def register(self, model, adapter_cls=VersionAdapter, **field_overrides):
+
+    def register(self, model=None, adapter_cls=VersionAdapter, signals=None, eager_signals=None, **field_overrides):
         """Registers a model with this revision manager."""
+        # Default to post_save if no signals are given
+        if signals is None and eager_signals is None:
+            signals = [post_save]
+        # Store signals for usage in the signal receiver
+        self._eager_signals[model] = list(eager_signals or [])
+        self._signals[model] = list(signals or [])
+        # Return a class decorator if model is not given
+        if model is None:
+            return partial(self.register, adapter_cls=adapter_cls, **field_overrides)
         # Prevent multiple registration.
         if self.is_registered(model):
             raise RegistrationError("{model} has already been registered with django-reversion".format(
                 model = model,
             ))
-        # Prevent proxy models being registered.
-        if model._meta.proxy:
-            raise RegistrationError(
-                "{model} is a proxy model, and cannot be used with django-reversion, register the parent class ({model_parent}) instead.".format(  # noqa
-                    model=model.__name__,
-                    model_parent=', '.join(
-                        [x.__name__ for x in model._meta.parents.keys()])
-                ))
         # Perform any customization.
         if field_overrides:
             adapter_cls = type(adapter_cls.__name__, (adapter_cls,), field_overrides)
         # Perform the registration.
         adapter_obj = adapter_cls(model)
         self._registered_models[self._registration_key_for_model(model)] = adapter_obj
-        # Connect to the post save signal of the model.
-        post_save.connect(self._post_save_receiver, model)
-    
+        # Connect to the selected signals of the model.
+        all_signals = self._signals[model] + self._eager_signals[model]
+        for signal in all_signals:
+            signal.connect(self._signal_receiver, model)
+        return model
+
     def get_adapter(self, model):
         """Returns the registration information for the given model class."""
         if self.is_registered(model):
@@ -394,7 +400,7 @@ class RevisionManager(object):
         raise RegistrationError("{model} has not been registered with django-reversion".format(
             model = model,
         ))
-        
+
     def unregister(self, model):
         """Removes a model from version control."""
         if not self.is_registered(model):
@@ -402,33 +408,38 @@ class RevisionManager(object):
                 model = model,
             ))
         del self._registered_models[self._registration_key_for_model(model)]
-        post_save.disconnect(self._post_save_receiver, model)
-    
+        all_signals = self._signals[model] + self._eager_signals[model]
+        for signal in all_signals:
+            signal.disconnect(self._signal_receiver, model)
+        del self._signals[model]
+        del self._eager_signals[model]
+
     def _follow_relationships(self, objects):
         """Follows all relationships in the given set of objects."""
         followed = set()
-        def _follow(obj):
-            if obj in followed or obj.pk is None:
+        def _follow(obj, exclude_concrete):
+            # Check the pk first because objects without a pk are not hashable
+            if obj.pk is None or obj in followed or (obj.__class__, obj.pk) == exclude_concrete:
                 return
             followed.add(obj)
             adapter = self.get_adapter(obj.__class__)
             for related in adapter.get_followed_relations(obj):
-                _follow(related)
+                _follow(related, exclude_concrete)
         for obj in objects:
-            _follow(obj)
+            exclude_concrete = None
+            if obj._meta.proxy:
+                exclude_concrete = (obj._meta.concrete_model, obj.pk)
+            _follow(obj, exclude_concrete)
         return followed
-    
+
     def _get_versions(self, db=None):
         """Returns all versions that apply to this manager."""
-        db = db or DEFAULT_DB_ALIAS
         return Version.objects.using(db).filter(
             revision__manager_slug = self._manager_slug,
         ).select_related("revision")
-        
+
     def save_revision(self, objects, ignore_duplicates=False, user=None, comment="", meta=(), db=None):
         """Saves a new revision."""
-        # Get the db alias.
-        db = db or DEFAULT_DB_ALIAS
         # Adapt the objects to a dict.
         if isinstance(objects, (list, tuple)):
             objects = dict(
@@ -491,16 +502,15 @@ class RevisionManager(object):
                 )
                 # Return the revision.
                 return revision
-    
+
     # Revision management API.
-    
+
     def get_for_object_reference(self, model, object_id, db=None):
         """
         Returns all versions for the given object reference.
-        
+
         The results are returned with the most recent versions first.
         """
-        db = db or DEFAULT_DB_ALIAS
         content_type = ContentType.objects.db_manager(db).get_for_model(model)
         versions = self._get_versions(db).filter(
             content_type = content_type,
@@ -515,19 +525,19 @@ class RevisionManager(object):
             versions = versions.filter(object_id=object_id)
         versions = versions.order_by("-pk")
         return versions
-    
+
     def get_for_object(self, obj, db=None):
         """
         Returns all the versions of the given object, ordered by date created.
-        
+
         The results are returned with the most recent versions first.
         """
         return self.get_for_object_reference(obj.__class__, obj.pk, db)
-    
+
     def get_unique_for_object(self, obj, db=None):
         """
         Returns unique versions associated with the object.
-        
+
         The results are returned with the most recent versions first.
         """
         versions = self.get_for_object(obj, db)
@@ -538,7 +548,7 @@ class RevisionManager(object):
                 changed_versions.append(version)
             last_serialized_data = version.serialized_data
         return changed_versions
-    
+
     def get_for_date(self, object, date, db=None):
         """Returns the latest version of an object for the given date."""
         versions = self.get_for_object(object, db)
@@ -549,14 +559,13 @@ class RevisionManager(object):
             raise Version.DoesNotExist
         else:
             return version
-    
+
     def get_deleted(self, model_class, db=None, model_db=None):
         """
         Returns all the deleted versions for the given model class.
-        
+
         The results are returned with the most recent versions first.
         """
-        db = db or DEFAULT_DB_ALIAS
         model_db = model_db or db
         content_type = ContentType.objects.db_manager(db).get_for_model(model_class)
         live_pk_queryset = model_class._default_manager.db_manager(model_db).all().values_list("pk", flat=True)
@@ -585,16 +594,28 @@ class RevisionManager(object):
             deleted_version_pks = list(deleted_version_pks)
         # Return the deleted versions!
         return self._get_versions(db).filter(pk__in=deleted_version_pks).order_by("-pk")
-        
+
     # Signal receivers.
-        
-    def _post_save_receiver(self, instance, **kwargs):
+
+    def _signal_receiver(self, instance, signal, **kwargs):
         """Adds registered models to the current revision, if any."""
         if self._revision_context_manager.is_active() and not self._revision_context_manager.is_managing_manually():
+            eager = signal in self._eager_signals[instance.__class__]
             adapter = self.get_adapter(instance.__class__)
-            version_data = lambda: adapter.get_version_data(instance, self._revision_context_manager._db)
-            self._revision_context_manager.add_to_context(self, instance, version_data)
+            if eager:
+                # pre_delete is a special case, because the instance will
+                # be modified by django right after this.
+                # don't use a lambda, but get the data out now.
+                version_data = adapter.get_version_data(instance, self._revision_context_manager._db)
+                self._revision_context_manager.add_to_context(self, copy.copy(instance), version_data)
+                for obj in self._follow_relationships([instance]):
+                    adapter = self.get_adapter(obj.__class__)
+                    version_data = adapter.get_version_data(obj, self._revision_context_manager._db)
+                    self._revision_context_manager.add_to_context(self, copy.copy(obj), version_data)
+            else:
+                version_data = lambda: adapter.get_version_data(instance, self._revision_context_manager._db)
+                self._revision_context_manager.add_to_context(self, instance, version_data)
+
 
-        
 # A shared revision manager.
 default_revision_manager = RevisionManager("default")
